@@ -34,7 +34,7 @@ struct _grn_ii {
   grn_io *seg;           /* I/O for a variety of segments */
   grn_io *chunk;         /* I/O for posting chunks */
   grn_obj *lexicon;      /* Lexicon table */
-  grn_obj_flags lflags;
+  grn_table_flags lflags;
   grn_encoding encoding; /* Character encoding */
                          /* This member is used for matching */
   uint32_t n_elements;   /* Number of elements in postings */
@@ -42,6 +42,7 @@ struct _grn_ii {
   struct grn_ii_header *header;
 };
 
+/* BGQ is buffer garbage queue? */
 #define GRN_II_BGQSIZE 16
 #define GRN_II_MAX_LSEG           0x10000
 #define GRN_II_W_TOTAL_CHUNK      40
@@ -63,8 +64,8 @@ struct grn_ii_header {
   uint32_t bgqtail;
   uint32_t bgqbody[GRN_II_BGQSIZE];
   uint32_t reserved[288];
-  uint32_t ainfo[GRN_II_MAX_LSEG];
-  uint32_t binfo[GRN_II_MAX_LSEG];
+  uint32_t ainfo[GRN_II_MAX_LSEG]; /* array info */
+  uint32_t binfo[GRN_II_MAX_LSEG]; /* buffer info */
   uint32_t free_chunks[GRN_II_N_CHUNK_VARIATION + 1];
   uint32_t garbages[GRN_II_N_CHUNK_VARIATION + 1];
   uint32_t ngarbages[GRN_II_N_CHUNK_VARIATION + 1];
